package de.fhtw.dataprofiler.dbconnectivity.handlers;

import java.sql.Connection;

import org.eclipse.core.commands.AbstractHandler;
import org.eclipse.core.commands.ExecutionEvent;
import org.eclipse.core.commands.ExecutionException;
import org.eclipse.core.resources.IFile;
import org.eclipse.core.runtime.IPath;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IEditorInput;
import org.eclipse.ui.IFileEditorInput;
import org.eclipse.ui.IWorkbenchWindow;
import org.eclipse.ui.PlatformUI;

import de.fhtw.dataprofiler.dbconnectivity.ActiveConnection;
import de.fhtw.dataprofiler.dbconnectivity.reverseengineerer.ReverseEngineerer;

/**
 * Reverse Engineering handler extends AbstractHandler, an IHandler base class.
 * 
 * @see org.eclipse.core.commands.IHandler
 * @see org.eclipse.core.commands.AbstractHandler
 */
public class ReverseEngineeringHandler extends AbstractHandler {
	/**
	 * The constructor.
	 */
	public ReverseEngineeringHandler() {
	}

	/**
	 * Extracts the active editor input file and
	 * starts the reverse engineering
	 * 
	 * the command has been executed, so extract extract the needed information
	 * from the application context.
	 */
	public Object execute(ExecutionEvent event) throws ExecutionException {

		//get connection from open wizard
		Connection connection = ActiveConnection.getInstance().getConnection();
		if (connection != null) {
			IWorkbenchWindow workbenchWindow = PlatformUI.getWorkbench().getActiveWorkbenchWindow();
			if (workbenchWindow != null){
				IEditorInput editorInput = workbenchWindow.getActivePage().getActiveEditor().getEditorInput();
				if (editorInput instanceof IFileEditorInput){
					//get active diagram file
					IFile diagramFile = ((IFileEditorInput)editorInput).getFile();
					//change file extension to 'sqlmodel'
					IPath modelFilePath = diagramFile.getFullPath().removeFileExtension().addFileExtension("sqlmodel");
					
					//start reverse engineering
					new ReverseEngineerer().startReverseEngineering(connection, modelFilePath);
				}
			}
		} else {
			MessageDialog.openError(Display.getCurrent().getActiveShell(), "No connection", "No active database connection has been found");
		}
		return null;
	}
}
