/**
 * 
 */
package de.fhtw.dataprofiler.profiling.analyses;

import java.sql.Connection;

import de.fhtw.dataprofiler.sqlmodel.schema.Schema;
import de.fhtw.dataprofiler.sqlmodel.tables.Column;
import de.fhtw.dataprofiler.sqlmodel.tables.Table;

/**
 * Class bundles all general profiling methods, which have to be executed
 * right after the reverse engineering 
 * 
 * @author Bjoern Knebel
 *
 */
public class DataProfilerImpl implements DataProfiler {
	
	private SchemaProfiler schemaProfiler;
	private TableProfiler tableProfiler;

	//Constructor
	//Sets connections
	public DataProfilerImpl(Connection connection){
		schemaProfiler = new SchemaProfiler(connection);
		tableProfiler = new TableProfiler(connection);
	}

	//Start overall profiling
	/* (non-Javadoc)
	 * @see de.fhtw.dataprofiler.profiling.analyses.DataProfiler#startProfiling(de.fhtw.dataprofiler.sqlmodel.schema.Schema)
	 */
	public void startProfiling(Schema schema){
		if (schema != null) {
			schemaProfiler.profileSchema(schema);
			
			//iterates through tables
			for (Table table : schema.getTables()) {
				tableProfiler.profileTable(table);	
			}
		}
	}	
}
