/**
 * 
 */
package de.fhtw.dataprofiler.profiling.analyses;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import de.fhtw.dataprofiler.sqlmodel.tables.Column;
import de.fhtw.dataprofiler.sqlmodel.tables.Table;

/**
 * Class for table profile analysis
 * - statistical metrics for columns of given table
 * 
 * @author Bjoern Knebel
 *
 */
public class TableProfiler {

	//Active connection
	private Connection conn;

	//Constructor
	//Sets active connection
	public TableProfiler(Connection connection) {
		this.conn = connection;
	}

	//Start method table profiling
	public void profileTable(Table table) {
		for (Column column : table.getColumns()) {
			countTotalValues(column);
			countDistinctValues(column);
			countNullValues(column);
			analyseMinValue(column);
			analyseMaxValue(column);
			analyseMinLength(column);
			analyseMaxLength(column);	
		}	
	}
	
	//Counts total values of the given column
	private void countTotalValues(Column column) {
		try {
			if (conn != null && !conn.isClosed()){
				String strStmt = "Select count(" + column.getName() + ") " + 
								 "from " + column.getTable().getName();
				
				Statement stmt = conn.createStatement();
				ResultSet rSet = stmt.executeQuery(strStmt);
				if (rSet.next())
					column.setTotalValues(rSet.getInt(1));
			}			
		} catch (SQLException e) {
			e.printStackTrace();
		}		
	}
	
	//Counts null values of the given column
	private void countNullValues(Column column) {
		try {
			if (conn != null && !conn.isClosed()){
				String strStmt = "Select count(*) from " + column.getTable().getName() + 
								 " where "+ column.getName() + " is null";
				
				Statement stmt = conn.createStatement();
				ResultSet rSet = stmt.executeQuery(strStmt);
				if (rSet.next())
					column.setNullValues(rSet.getInt(1));
			}			
		} catch (SQLException e) {
			e.printStackTrace();
		}		
	}
	
	//Counts distinct values of the given column
	private void countDistinctValues(Column column) {
		try {
			if (conn != null && !conn.isClosed()){
				String strStmt = "Select count(distinct " + column.getName() + ")" +
								 " from  " + column.getTable().getName();

				Statement stmt = conn.createStatement();	
				ResultSet rSet = stmt.executeQuery(strStmt);
				if (rSet.next())
					column.setDistinctValues(rSet.getInt(1));
				stmt.close();
			}			
		} catch (SQLException e) {
			e.printStackTrace();
		}		
	}
	
	//Determines the min value of the given column
	private void analyseMinValue(Column column) {
		try {
			if (conn != null && !conn.isClosed()){
				if (column.getType().equals("blob")){
					
					//if type 'blob' the value can't be serialized
					column.setMinValue("n/a");
				} else {
					
					//determine min value of given column
					String strStmt = "Select min(" + column.getName() + ")" +
	 				 " from  " + column.getTable().getName();

					Statement stmt = conn.createStatement();	
					ResultSet rSet = stmt.executeQuery(strStmt);
					if (rSet.next())
						column.setMinValue(rSet.getString(1));
					stmt.close();
				}
			}			
		} catch (SQLException e) {
			e.printStackTrace();
		}		
	}
	
	//Determines the max value of the given column
	private void analyseMaxValue(Column column) {
		try {
			if (conn != null && !conn.isClosed()){
				if (column.getType().equals("blob")){
					
					//if type 'blob' the value can't be serialized
					column.setMaxValue("n/a");
				} else {
					
					//determine max value of given column
					String strStmt = "Select max(" + column.getName() + ")" +
									 " from  " + column.getTable().getName();
	
					Statement stmt = conn.createStatement();	
					ResultSet rSet = stmt.executeQuery(strStmt);
					if (rSet.next())
						column.setMaxValue(rSet.getString(1));
					stmt.close();
				}
			}			
		} catch (SQLException e) {
			e.printStackTrace();
		}		
	}
	
	//Determines the min length of the given column
	private void analyseMinLength(Column column) {
		try {
			if (conn != null && !conn.isClosed()){
				String strStmt = "Select min(length(" + column.getName() + "))" +
				 				 " from  " + column.getTable().getName();

				// TODO function length() probably doesn't work for all databases
				// implementation of a generic solution preferable
				Statement stmt = conn.createStatement();	
				ResultSet rSet = stmt.executeQuery(strStmt);
				if (rSet.next())
					column.setMinLength(rSet.getInt(1));
				stmt.close();
			}			
		} catch (SQLException e) {
			e.printStackTrace();
		}		
	}
	
	//Determines the max length of the given column
	private void analyseMaxLength(Column column) {
		try {
			if (conn != null && !conn.isClosed()){
				String strStmt = "Select max(length(" + column.getName() + "))" +
								 " from  " + column.getTable().getName();

				// TODO function length() probably doesn't work for all databases
				// implementation of a generic solution preferable
				Statement stmt = conn.createStatement();	
				ResultSet rSet = stmt.executeQuery(strStmt);
				if (rSet.next())
					column.setMaxLength(rSet.getInt(1));
				stmt.close();
			}			
		} catch (SQLException e) {
			e.printStackTrace();
		}
	}
	
}
