package de.fhtw.dataprofiler.sqlmodel.diagram.profile.actions;

import org.eclipse.gef.EditPart;
import org.eclipse.gmf.runtime.notation.View;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.jface.viewers.ISelection;
import org.eclipse.jface.viewers.IStructuredSelection;
import org.eclipse.swt.widgets.Display;
import org.eclipse.ui.IActionDelegate;
import org.eclipse.ui.IObjectActionDelegate;
import org.eclipse.ui.IWorkbenchPart;

import de.fhtw.dataprofiler.sqlmodel.diagram.profile.analyses.PatternAnalysis;
import de.fhtw.dataprofiler.sqlmodel.diagram.profile.dialogs.PatternAnalysisDialog;
import de.fhtw.dataprofiler.sqlmodel.tables.Column;

/**
 * Launches a pattern analysis for the selected column and
 * displays a proper dialog with the results
 * 
 * @author Bjoern Knebel 
 * 
 */
public class LaunchPatternAnalysis implements IObjectActionDelegate {

	private Column selectedColumn;

	/**
	 * Constructor for LaunchPatternAnalysis.
	 */
	public LaunchPatternAnalysis() {
		super();
	}

	/**
	 * @see IObjectActionDelegate#setActivePart(IAction, IWorkbenchPart)
	 */
	public void setActivePart(IAction action, IWorkbenchPart targetPart) {
	}

	/**
	 * @see IActionDelegate#run(IAction)
	 */
	public void run(IAction action) {

		PatternAnalysis patternAnalysis = new PatternAnalysis();
		if (patternAnalysis.analysePatterns(selectedColumn)) {

			PatternAnalysisDialog dialog = new PatternAnalysisDialog(Display.getCurrent().getActiveShell(),
					patternAnalysis.getPatterns(),selectedColumn.getTotalValues());
			dialog.open();
		}else {
			MessageDialog.openError(Display.getCurrent().getActiveShell(), "No pattern", "No pattern has been found");
		}
	}

	/**
	 * Sets the currently selected column
	 * 
	 * @see IActionDelegate#selectionChanged(IAction, ISelection)
	 */
	public void selectionChanged(IAction action, ISelection selection) {
		Object selPart = ((IStructuredSelection) selection).getFirstElement();
		Object model = (selPart instanceof EditPart) ? ((EditPart) selPart).getModel() : null;
		if (model != null) {
			View view = (View) model;
			selectedColumn = (Column) view.getElement();
		}
	}

}
