package de.fhtw.dataprofiler.sqlmodel.diagram.profile.analyses;

import java.util.Comparator;

/**
 * Encapsulates the pattern string and its frequency
 * 
 * @author Bjoern Knebel
 */
public class Pattern implements Comparator {

    private String pattern;
    private int count;

    /**
     * No arg constructor for Pattern. Sets inital frequency to 0.
     */
    public Pattern() {
        count = 0;
    }

    /**
     * Stores a int and sets initial freuqncy to 1.
     * 
     * @param pattern
     *            The int to store.
     */
    public Pattern(String pattern) {
        this.pattern = pattern;
        count = 1;
    }

    /**
     * @return Returns the count.
     */
    public int getCount() {
        return count;
    }

    /**
     * @param count
     *            The count to set.
     */
    public void setCount(int count) {
        this.count = count;
    }

    /**
     * @return Returns the Pattern.
     */
    public String getPattern() {
        return pattern;
    }

    /**
     * @param pattern
     *            The pattern to set.
     */
    public void setPattern(String pattern) {
        this.pattern = pattern;
    }

    /**
     * Generates a String representation of this Pattern object.
     * @return the pattern and its frequency.
     */
    public String toString() {
        return pattern + "\t\t" + count;
    }

    /**
     * Increments the frequency for this pattern. 
     */
    public void increment() {
        count++;
    }

    /**
     * Imposes a total ordering on Pattern objects. The ordering is descending 
     * frequency.
     */
    public int compare(Object arg0, Object arg1) {
        return ((Pattern) arg1).count - ((Pattern) arg0).count;
    }
}