/**
 * 
 */
package de.fhtw.dataprofiler.sqlmodel.diagram.profile.analyses;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.HashMap;
import java.util.Map;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.widgets.Display;

import de.fhtw.dataprofiler.dbconnectivity.ActiveConnection;
import de.fhtw.dataprofiler.sqlmodel.tables.Column;

/**
 * Class analysis patterns and increments their frequency
 * 
 * @author Bjoern Knebel
 *
 */
public class PatternAnalysis {
	
	HashMap<String,Pattern> patterns = new HashMap<String,Pattern>();
	
	
	/**
	 * Analysis patterns for given column
	 * 
	 * @param column
	 * @return success
	 */
	public boolean analysePatterns(Column column){
		
		//get active connection
		Connection conn = ActiveConnection.getInstance().getConnection();

		try {
			if(conn != null && !conn.isClosed()){
				//Get all values of the column
				String strStmt = "Select " + column.getName() + 
								 " from " + column.getTable().getName(); 
				
				Statement stmt = conn.createStatement();
				ResultSet rSet = stmt.executeQuery(strStmt);
				while(rSet.next()){
					//stores the analysed pattern of a string
					storePattern(getPattern(rSet.getString(1)));
				}
				if(patterns.size() > 0)
					//Analysis successful
					return true;
			} else {
				MessageDialog.openError(Display.getCurrent().getActiveShell(), "No connection", "No active database connection has been found");
			}
		} catch (SQLException e) {
			e.printStackTrace();
		}
		return false;
	}
	
	
	/**
	 * Stores the pattern object to a HashMap
	 * 
	 * @param strPattern
	 */
	private void storePattern(String strPattern) {
		Pattern myPattern = patterns.get(strPattern);
		if (myPattern == null) {
			//first pattern of this kind
		    patterns.put(strPattern, new Pattern(strPattern));
		} else {
			//pattern is already contained
		    myPattern.increment();
		    patterns.put(strPattern, myPattern);
		}
	}

	/**
	 * Changes an arbitrary string value to an abstract pattern
	 * 
	 * @param testString
	 * @return
	 */
	private String getPattern(String testString){
		char[] testArray = testString.toCharArray();
		for (int i = 0; i < testArray.length; i++) {
			if (Character.isLetter(testArray[i])) {
				if (Character.isUpperCase(testArray[i])) {
					testArray[i] = 'A';
				} else {
					testArray[i] = 'a';
				}
			} else if (Character.isDigit(testArray[i])){
				testArray[i] = '9';
			}
		}
		return String.valueOf(testArray);
	}
	
	/**
	 * @return HashMap of patterns
	 */
	public Map<String,Pattern> getPatterns(){

		return patterns;
	}
}
