/**
 * 
 */
package de.fhtw.dataprofiler.sqlmodel.diagram.profile.dialogs;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;

import org.eclipse.jface.dialogs.Dialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.TableColumn;
import org.eclipse.swt.widgets.TableItem;

import de.fhtw.dataprofiler.sqlmodel.diagram.profile.ProfilerUtils;
import de.fhtw.dataprofiler.sqlmodel.diagram.profile.analyses.Pattern;

/**
 * Displays a dialog with a table of the pattern distribution
 * 
 * @author Bjoern Knebel
 * 
 */
public class PatternAnalysisDialog extends Dialog {

	private List<Pattern> listPatterns = new ArrayList<Pattern>();
	private int totalValues = 0;

	public PatternAnalysisDialog(Shell parentShell, Map<String, Pattern> patterns, int totalValues) {
		super(parentShell);
		listPatterns.addAll(patterns.values());
		this.totalValues = totalValues;
	}

	protected Control createDialogArea(Composite parent) {		
		Composite comp = (Composite) super.createDialogArea(parent);
		comp.setSize(300,300);
		
		//Set dialog title
		getShell().setText("Pattern Analysis");
		
		//Define Layout
		GridData gd = new GridData(	GridData.FILL_BOTH | 
									GridData.GRAB_HORIZONTAL | 
									GridData.GRAB_VERTICAL);

		Table table = new Table(comp, SWT.BORDER);
		table.setHeaderVisible(true);
		table.setLayoutData(gd);

		//Declare Columns
		TableColumn columnPattern = new TableColumn(table, SWT.NONE);
		columnPattern.setText("Pattern");

		TableColumn columnCount = new TableColumn(table, SWT.NONE);
		columnCount.setText("Count");

		TableColumn columnPer = new TableColumn(table, SWT.NONE);
		columnPer.setText("%");

		// Sort pattern by frequency using implemented compare()
		Collections.sort(listPatterns, new Pattern());

		//Set cells
		int i = 0;
		for (Pattern p : listPatterns) {
			TableItem item = new TableItem(table, SWT.NONE);
			item.setText(0, p.getPattern());
			item.setText(1, String.valueOf(p.getCount()));
			item.setText(2, ProfilerUtils.getPercentage(totalValues,p.getCount()));
			i++;
			if(i==10) break;
		}

		// Resize columns
		columnPattern.pack();
		columnCount.pack();
		columnPer.pack();

		return comp;
	}

	protected int getShellStyle() {
		int ret = super.getShellStyle();
		return ret | SWT.RESIZE;
	}
}
